==> kafka-3.7.2-src.tgz: kafka-3.7.2-src/connect/runtime/src/main/java/org/apache/kafka/connect/util/Callback.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.kafka.connect.util;

/**
 * Generic interface for callbacks
 */
public interface Callback<V> {
    /**
     * Invoked upon completion of the operation.
     *
     * @param error the error that caused the operation to fail, or null if no error occurred
     * @param result the return value, or null if the operation failed
     */
    void onCompletion(Throwable error, V result);

    default void recordStage(Stage stage) {
    }

    default <V2> Callback<V2> chainStaging(Callback<V2> chained) {
        return new Callback<V2>() {
            @Override
            public void recordStage(Stage stage) {
                Callback.this.recordStage(stage);
            }

            @Override
            public void onCompletion(Throwable error, V2 result) {
                chained.onCompletion(error, result);
            }
        };
    }

}

==> kafka-4.0.1-src.tgz: kafka-4.0.1-src/connect/runtime/src/main/java/org/apache/kafka/connect/util/Callback.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.kafka.connect.util;

/**
 * Generic interface for callbacks
 */
public interface Callback<V> {
    /**
     * Invoked upon completion of the operation.
     *
     * @param error the error that caused the operation to fail, or null if no error occurred
     * @param result the return value, or null if the operation failed
     */
    void onCompletion(Throwable error, V result);

    default void recordStage(Stage stage) {
    }

    default <V2> Callback<V2> chainStaging(Callback<V2> chained) {
        return new Callback<>() {
            @Override
            public void recordStage(Stage stage) {
                Callback.this.recordStage(stage);
            }

            @Override
            public void onCompletion(Throwable error, V2 result) {
                chained.onCompletion(error, result);
            }
        };
    }

}
